package com.atlassian.elasticsearch.buckler;

import com.atlassian.elasticsearch.buckler.config.BucklerConfig;
import com.atlassian.elasticsearch.buckler.config.TlsConfig;
import io.netty.channel.Channel;
import io.netty.channel.ChannelHandler;
import io.netty.channel.ChannelPipeline;
import io.netty.handler.ssl.SslHandler;
import org.elasticsearch.common.network.NetworkService;
import org.elasticsearch.common.settings.Settings;
import org.elasticsearch.common.util.BigArrays;
import org.elasticsearch.common.util.concurrent.ThreadContext;
import org.elasticsearch.common.xcontent.NamedXContentRegistry;
import org.elasticsearch.http.HttpHandlingSettings;
import org.elasticsearch.http.netty4.Netty4HttpServerTransport;
import org.elasticsearch.threadpool.ThreadPool;

/**
 * Basic authentication and TLS for HTTP transport.
 */
public class SecureHttpServerTransport extends Netty4HttpServerTransport {

    private final BucklerConfig config;

    public SecureHttpServerTransport(Settings settings, NetworkService networkService,
                                     BigArrays bigArrays, BucklerConfig config,
                                     ThreadPool threadPool, NamedXContentRegistry xContentRegistry,
                                     Dispatcher dispatcher) {
        super(settings, networkService, bigArrays, threadPool, xContentRegistry, dispatcher);
        this.config = config;
    }

    @Override
    public ChannelHandler configureServerChannelHandler() {
        return config.getTlsConfig().isEnabledForHttp()
                ? new TlsHttpChannelPipelineFactory(this, handlingSettings, config.getTlsConfig(), super.configureServerChannelHandler())
                : super.configureServerChannelHandler();
    }

    private static class TlsHttpChannelPipelineFactory extends HttpChannelHandler {

        private final ChannelHandler channelHandler;
        private final TlsConfig tlsConfig;

        public TlsHttpChannelPipelineFactory(Netty4HttpServerTransport transport, HttpHandlingSettings handlingSettings, TlsConfig tlsConfig, ChannelHandler channelHandler) {
            super(transport, handlingSettings);
            this.tlsConfig = tlsConfig;
            this.channelHandler = channelHandler;
        }

        @Override
        protected void initChannel(Channel ch) throws Exception {
            ChannelPipeline pipeline = ch.pipeline();

            SslHandler sslHandler = tlsConfig.createHandler(false);
            pipeline.addFirst("tls", sslHandler);
            pipeline.addLast(channelHandler);
        }
    }
}
