package com.atlassian.elasticsearch.buckler.config;

import org.elasticsearch.common.settings.Settings;

public class BruteForceProtectionSettings {
    private final boolean bruteForceProtectionEnabled;
    private final boolean clientLockoutEnabled;
    private final boolean globalLockoutEnabled;
    private final int maxClientFailures;
    private final int maxGlobalFailures;
    private final int timeoutPeriod;

    BruteForceProtectionSettings(Settings settings) {
        this.maxClientFailures = settings.getAsInt("auth.lockout.perclient.attempts", 25);
        this.maxGlobalFailures = settings.getAsInt("auth.lockout.global.attempts", 100);
        this.timeoutPeriod = settings.getAsInt("auth.lockout.attempts.reset", 5);
        this.bruteForceProtectionEnabled = settings.getAsBoolean("auth.lockout.enabled", true);
        this.clientLockoutEnabled = settings.getAsBoolean("auth.lockout.perclient.enabled", true);
        this.globalLockoutEnabled = settings.getAsBoolean("auth.lockout.global.enabled", true);
    }

    public int getMaxClientFailures() {
        return maxClientFailures;
    }

    public int getMaxGlobalFailures() {
        return maxGlobalFailures;
    }

    public int getTimeoutPeriod() {
        return timeoutPeriod;
    }

    public boolean isBruteForceProtectionEnabled() {
        return bruteForceProtectionEnabled;
    }

    public boolean isClientLockoutEnabled() {
        return clientLockoutEnabled;
    }

    public boolean isGlobalLockoutEnabled() {
        return globalLockoutEnabled;
    }
}
